@@ -7,7 +7,7 @@ class FaceDetBase(object):
     __metaclass__ = ABCMeta
 
     @abstractmethod
-    def detect(self, *args, **kwargs) -> Any:
+    def apply_detecting(self, *args, **kwargs) -> Any:
         raise NotImplementedError
 
 
@@ -15,5 +15,5 @@ class LandmarksDetBase(object):
     __metaclass__ = ABCMeta
 
     @abstractmethod
-    def detect(self, *args, **kwargs) -> Any:
+    def apply_detecting(self, *args, **kwargs) -> Any:
         raise NotImplementedError
