@@ -138,13 +138,13 @@ class _PIPNetImpl(nn.Module, LandmarksTrainableBase):
 
                     self.meanface_status = True
 
-    def loss(self, *args, **kwargs) -> _PIPNet_Loss_Output_Type:
-        return _loss_impl(*args, **kwargs)
+    def apply_losses(self, *args, **kwargs) -> _PIPNet_Loss_Output_Type:
+        return _losses_impl(*args, **kwargs)
 
-    def detect(self, image: np.ndarray) -> np.ndarray:
-        return _detect_impl(net=self, image=image)
+    def apply_detecting(self, image: np.ndarray) -> np.ndarray:
+        return _detecting_impl(net=self, image=image)
 
-    def training(
+    def apply_training(
             self,
             annotation_path: str,
             criterion_cls: nn.Module = nn.MSELoss(),
@@ -152,12 +152,12 @@ class _PIPNetImpl(nn.Module, LandmarksTrainableBase):
             learning_rate: float = 0.0001,
             cls_loss_weight: float = 10.,
             reg_loss_weight: float = 1.,
-            num_nb: int = 10,
             num_epochs: int = 60,
             save_dir: Optional[str] = "./save",
             save_interval: Optional[int] = 10,
             save_prefix: Optional[str] = "",
             decay_steps: Optional[List[int]] = (30, 50),
+            logging_interval: Optional[int] = 1,
             decay_gamma: Optional[float] = 0.1,
             device: Optional[Union[str, torch.device]] = "cuda",
             transform: Optional[transforms.LandmarksCompose] = None,
@@ -176,7 +176,6 @@ class _PIPNetImpl(nn.Module, LandmarksTrainableBase):
         :param learning_rate: learning rate, default 0.0001
         :param cls_loss_weight: weight for heatmap classification
         :param reg_loss_weight: weight for offsets regression
-        :param num_nb: the number of Nearest-neighbor landmarks for NRM, default 10
         :param num_epochs: the number of training epochs
         :param save_dir: the dir to save checkpoints
         :param save_interval: the interval to save checkpoints
@@ -185,12 +184,14 @@ class _PIPNetImpl(nn.Module, LandmarksTrainableBase):
         :param decay_steps: decay steps for learning rate scheduler
         :param decay_gamma: decay gamma for learning rate scheduler
         :param device: training device, default cuda.
+        :param logging_interval: iter interval for logging.
         :param transform: user specific transform. If None, torchlm will build a default transform,
          more details can be found at `torchlm.transforms.build_default_transform`
-        :param coordinates_already_normalized: denoted the label in annotation_path is normalized(by image size) of not
+        :param coordinates_already_normalized: denoted the label in annotation_path is normalized(by image size) or not
         :param kwargs:  params for DataLoader
         :return: A trained model.
         """
+        print("Parameters for DataLoader: ", kwargs)
         device = device if torch.cuda.is_available() else "cpu"
         # prepare dataset
         default_dataset = _PIPTrainDataset(
@@ -212,22 +213,24 @@ class _PIPNetImpl(nn.Module, LandmarksTrainableBase):
             learning_rate=learning_rate,
             cls_loss_weight=cls_loss_weight,
             reg_loss_weight=reg_loss_weight,
-            num_nb=num_nb,
+            num_nb=self.num_nb,
             num_epochs=num_epochs,
             save_dir=save_dir,
             save_prefix=save_prefix,
             save_interval=save_interval,
             decay_steps=decay_steps,
             decay_gamma=decay_gamma,
-            device=device
+            device=device,
+            logging_interval=logging_interval
         )
 
-    def evaluating(
+    def apply_evaluating(
             self,
             annotation_path: str,
             norm_indices: List[int] = (60, 72),
             dataset_type: Optional[str] = None,
-            coordinates_already_normalized: Optional[bool] = False
+            coordinates_already_normalized: Optional[bool] = False,
+            eval_normalized_coordinates: Optional[bool] = False
     ) -> Tuple[float, float, float]:  # NME, FR, AUC
         # prepare dataset
         eval_dataset = _PIPEvalDataset(
@@ -239,17 +242,18 @@ class _PIPNetImpl(nn.Module, LandmarksTrainableBase):
             net=self,
             eval_dataset=eval_dataset,
             norm_indices=norm_indices,
-            dataset_type=dataset_type
+            dataset_type=dataset_type,
+            eval_normalized_coordinates=eval_normalized_coordinates
         )
 
-    def export(
+    def apply_exporting(
             self,
             onnx_path: str = "./onnx/pipnet.onnx",
             opset: int = 12,
             simplify: bool = False,
             output_names: Optional[List[str]] = None
     ) -> None:
-        _export_impl(
+        _exporting_impl(
             net=self,
             onnx_path=onnx_path,
             opset=opset,
@@ -257,12 +261,15 @@ class _PIPNetImpl(nn.Module, LandmarksTrainableBase):
             output_names=output_names
         )
 
+    def apply_freezing(self, *args, **kwargs) -> Any:
+        raise NotImplementedError
+
     def forward(self, *args, **kwargs) -> _PIPNet_Output_Type:
         raise NotImplementedError
 
 
 @torch.no_grad()
-def _detect_impl(
+def _detecting_impl(
         net: _PIPNetImpl,
         image: np.ndarray
 ) -> np.ndarray:
@@ -281,7 +288,7 @@ def _detect_impl(
 
     height, width, _ = image.shape
     image: np.ndarray = cv2.resize(image, (net.input_size, net.input_size))  # 256, 256
-    image: Tensor = torch.from_numpy(_normalize(img=image)).unsqueeze(0)  # (1,3,256,256)
+    image: Tensor = torch.from_numpy(_normalize(img=image)).contiguous().unsqueeze(0)  # (1,3,256,256)
     outputs_cls, outputs_x, outputs_y, outputs_nb_x, outputs_nb_y = net.forward(image)
     # (1,68,8,8)
     tmp_batch, tmp_channel, tmp_height, tmp_width = outputs_cls.size()
@@ -307,13 +314,19 @@ def _detect_impl(
     outputs_nb_y_select = outputs_nb_y_select.squeeze(1).view(-1, net.num_nb)  # (68,10)
 
     # tmp_width=tmp_height=8 max_ids->[0,63] calculate grid center (cx,cy) in 8x8 map
+    # https://github.com/jhb86253817/PIPNet/issues/4
+    # lms_pred_x = (max_ids % tmp_width).view(-1, 1).float() + outputs_x_select.view(-1, 1)  # x=cx+offset_x
+    # lms_pred_y = (max_ids // tmp_width).view(-1, 1).float() + outputs_y_select.view(-1, 1)  # y=cy+offset_y
     lms_pred_x = (max_ids % tmp_width).view(-1, 1).float() + outputs_x_select.view(-1, 1)  # x=cx+offset_x
-    lms_pred_y = (max_ids // tmp_width).view(-1, 1).float() + outputs_y_select.view(-1, 1)  # y=cy+offset_y
+    lms_pred_y = torch.floor(max_ids / tmp_width).view(-1, 1).float() + outputs_y_select.view(-1, 1)  # y=cy+offset_y
     lms_pred_x /= 1.0 * net.input_size / net.net_stride  # normalize coord (x*32)/256
     lms_pred_y /= 1.0 * net.input_size / net.net_stride  # normalize coord (y*32)/256
 
+    # lms_pred_nb_x = (max_ids % tmp_width).view(-1, 1).float() + outputs_nb_x_select  # (68,10)
+    # lms_pred_nb_y = (max_ids // tmp_width).view(-1, 1).float() + outputs_nb_y_select  # (68,10)
+    # https://github.com/jhb86253817/PIPNet/issues/4
     lms_pred_nb_x = (max_ids % tmp_width).view(-1, 1).float() + outputs_nb_x_select  # (68,10)
-    lms_pred_nb_y = (max_ids // tmp_width).view(-1, 1).float() + outputs_nb_y_select  # (68,10)
+    lms_pred_nb_y = torch.floor(max_ids / tmp_width).view(-1, 1).float() + outputs_nb_y_select  # (68,10)
     lms_pred_nb_x = lms_pred_nb_x.view(-1, net.num_nb)  # (68,10)
     lms_pred_nb_y = lms_pred_nb_y.view(-1, net.num_nb)  # (68,10)
     lms_pred_nb_x /= 1.0 * net.input_size / net.net_stride  # normalize coord (nx*32)/256
@@ -327,13 +340,14 @@ def _detect_impl(
     lms_pred_merge = torch.cat((tmp_x, tmp_y), dim=1)  # (68,2)
     lms_pred_merge = lms_pred_merge.cpu().numpy()  # (68,2)
 
-    lms_pred_merge[:, 0] *= float(width)
-    lms_pred_merge[:, 1] *= float(height)
+    # de-normalize
+    lms_pred_merge[:, 0] *= float(width)  # e.g 256
+    lms_pred_merge[:, 1] *= float(height)  # e.g 256
 
     return lms_pred_merge
 
 
-def _loss_impl(
+def _losses_impl(
         outputs_cls: Tensor,
         outputs_x: Tensor,
         outputs_y: Tensor,
@@ -412,6 +426,7 @@ def _training_impl(
         save_dir: Optional[str] = "./save",
         save_prefix: Optional[str] = "",
         save_interval: Optional[int] = 10,
+        logging_interval: Optional[int] = 1,
         decay_steps: Optional[List[int]] = (30, 50),
         decay_gamma: Optional[float] = 0.1,
         device: Optional[Union[str, torch.device]] = "cuda"
@@ -425,6 +440,7 @@ def _training_impl(
             f"running PIPNet ..."
         )
 
+    net = net.to(device)
     optimizer = optim.Adam(net.parameters(), lr=learning_rate)
     scheduler = optim.lr_scheduler.MultiStepLR(
         optimizer=optimizer,
@@ -453,7 +469,7 @@ def _training_impl(
             labels_nb_x = labels_nb_x.to(device)
             labels_nb_y = labels_nb_y.to(device)
             outputs_cls, outputs_x, outputs_y, outputs_nb_x, outputs_nb_y = net(inputs)
-            loss_cls, loss_x, loss_y, loss_nb_x, loss_nb_y = net.loss(
+            loss_cls, loss_x, loss_y, loss_nb_x, loss_nb_y = net.apply_losses(
                 outputs_cls=outputs_cls,
                 outputs_x=outputs_x,
                 outputs_y=outputs_y,
@@ -476,7 +492,7 @@ def _training_impl(
             loss.backward()
             optimizer.step()
 
-            if i % 10 == 0:
+            if i % logging_interval == 0:
                 print(
                     '[Epoch {:d}/{:d}, Batch {:d}/{:d}] <Total loss: {:.6f}> <cls loss: {:.6f}> '
                     '<x loss: {:.6f}> <y loss: {:.6f}> <nbx loss: {:.6f}> <nby loss: {:.6f}>'
@@ -507,19 +523,23 @@ def _training_impl(
     return net
 
 
+@torch.no_grad()
 def _evaluating_impl(
         net: _PIPNetImpl,
         eval_dataset: _PIPEvalDataset,
         norm_indices: List[int] = (60, 72),
         dataset_type: Optional[str] = None,
+        eval_normalized_coordinates: Optional[bool] = False
 ) -> Tuple[float, float, float]:
     """
     :param net: PIPNet instance
     :param eval_dataset: _PIPEvalDataset instance
     :param norm_indices: the indexes of two eyeballs.
     :param dataset_type: optional, specific dataset, e.g WFLW/COFW/300W
+    :param eval_normalized_coordinates: evaluating on normalized coordinates
     :return: NME, FR, AUC
     """
+    net.eval()
     import tqdm
     norm_indices = list(norm_indices)
     if dataset_type is not None:
@@ -537,8 +557,20 @@ def _evaluating_impl(
 
     nmes = []
     # evaluating
-    for image, lms_gt in tqdm.tqdm(eval_dataset, colour="green"):
-        lms_pred = net.detect(image=image)  # (n,2)
+    for image, lms_gt in tqdm.tqdm(
+            eval_dataset,
+            colour="YELLOW",
+            desc="Evaluating PIPNet"
+    ):
+        lms_pred = net.apply_detecting(image=image)  # (n,2)
+
+        if eval_normalized_coordinates:
+            h, w, _ = image.shape  # e.g 256, 256
+            lms_pred[:, 0] /= float(w)
+            lms_pred[:, 1] /= float(h)
+            lms_gt[:, 0] /= float(w)
+            lms_gt[:, 1] /= float(h)
+
         if norm_indices is not None:
             norm = np.linalg.norm(lms_gt[norm_indices[0]] - lms_gt[norm_indices[1]])
         else:
@@ -551,7 +583,7 @@ def _evaluating_impl(
     return nme, fr, auc
 
 
-def _export_impl(
+def _exporting_impl(
         net: _PIPNetImpl,
         onnx_path: str = "./onnx/pipnet.onnx",
         opset: int = 12,
